#! /usr/bin/perl -w
use strict;
##############################################################################################
####   Script for ELF/NCI analysis - Robin Chaudret - April 2013                          ####
####   When using this script please cite:                                                ####
####   "Coupling quantum interpretative techniques: another look at chemical mechanisms   ####
####   in organic reactions"                                                              ####
####   N. Gillet, R. Chaudret, J. Contreras-Garcia, W. Yang, B. Silvi, J.-P. Piquemal     ####
####   JCTC, 2012, 8, 3993-3997                                                           ####
##############################################################################################
#### This script allow to create the ELF and/or NCI picture of the reaction mechanism.    ####
#### To launch it: ./elf_nci_visualization.pl Input Ouput "name of the vmd input"         ####
#### Input KEYWORDS                                                                       ####
####  - NAME : name of the different elf and/or nci files to treat (ex: geom_)            #### 
####  - GEOM : number of the first and last structure to consider (ex: 0 10)              ####
####  - REPRESENTATION : choice of the representation used by VMD (CPK, licorice or line) ####
####  - ELF : do ELF calculation and ELF isovalue (ex: ELF 0.5),                          ####
####  - NCI : do NCI calculation and RDG isovalue, rho min and max (ex: NCI 0.3 -2.0 2.0) ####
####  - set viewpoints... viewpoint desired taken from VMD                                ####
##############################################################################################




#######################################################
#### Open input file and output files (vmd inputs) ####
#######################################################

if (!defined $ARGV[0]) {
    printf STDERR "no input file specified\n";
    printf STDERR "./elf_nci_visualization.pl Input\n";
    exit;
} else {
open INPUT, $ARGV[0] or die "Cannot Open $ARGV[0]:$!\n";
}

if (!defined $ARGV[1]) {
    printf STDERR "no output file specified\n";
    printf STDERR "output file = output\n";
    open OUTFILE, ">output" or die "Cannot Open output:$!\n";
    printf OUTFILE "no output file specified\n";
    printf OUTFILE "output file = output\n";
} else {
    open OUTFILE, ">$ARGV[1]" or die "Cannot Open $ARGV[1]:$!\n";
    printf OUTFILE "output file = $ARGV[1]\n";
}

if (!defined $ARGV[2]) {
    printf STDERR "no vmd surface file specified\n";
    printf STDERR "vmd surface input = vmd_surface.vmd\n";
    printf OUTFILE "no vmd surface file specified\n";
    printf OUTFILE "vmd surface input = vmd_surface.vmd\n";
    open OUTPUT, ">vmd_surface.vmd" or die "Cannot Open vmd_surface.vmd:$!\n";
} else {
    printf OUTFILE "vmd surface input = $ARGV[2]\n";
    open OUTPUT, ">$ARGV[2]" or die "Cannot Open $ARGV[2]:$!\n";
}

if (!defined $ARGV[3]) {
    printf STDERR "no vmd attractor file specified\n";
    printf STDERR "vmd attractor input = vmd_attractor.vmd\n";
    printf OUTFILE "no vmd attractor file specified\n";
    printf OUTFILE "vmd attractor input = vmd_attractor.vmd\n";
    open OUTPUT2, ">vmd_attractor.vmd" or die "Cannot Open vmd_attractor.vmd:$!\n";
} else {
    open OUTPUT2, ">$ARGV[3]" or die "Cannot Open $ARGV[3]:$!\n";
    printf OUTFILE "vmd attractor input = $ARGV[3]\n";
}



##############################
#### Define the variables ####
##############################

my $first=0;
my $last=0;
my $step=0;
my @line_info;
my @line_info_2;
my @line_info_3;
my $isoelf;
my $isoelf_value;
my @line_info_elf;
my @line_info_nci;
my $isonci;
my $isonci_value;
my $scalencimin = 0;
my $scalencimax = 0;
my $view;
my @viewpoint;
my $type;
my $rep;
my $name;
my $name_input=0;
my $rep_input=0;
my $view_input=0;
my $elf_cutoff=0;
my $rdg_cutoff=0;
my $rho_min=0;
my $rho_max=0;
my $do_elf=0;
my $do_nci=0;
my $do_both=0;



########################################
#### Get parameters from input file ####
########################################

while (my $line = <INPUT>) {

    @line_info = split/\s+/, $line;

    if ($line_info[0] eq "NAME") {
        $name = $line_info[1];
        $name_input = 1;
        printf OUTFILE "looking for all the $name files\n"
    } elsif ($line_info[0] eq "GEOM") {
        $first = $line_info[1];
        $last = $line_info[2];
        $step = $line_info[3];
        printf OUTFILE "going from $first to $last geometries\n"
    } elsif ($line_info[0] eq "REPRESENTATION") {
        $rep = $line_info[1];
        $rep_input = 1;
        printf OUTFILE "$rep representation will be used for the atoms\n"
    } elsif ($line_info[0] eq "ELF") {
        $do_elf = 1;
        $elf_cutoff = $line_info[1];
    } elsif ($line_info[0] eq "NCI") {
        $do_nci = 1;
        $rdg_cutoff = $line_info[1];
        $rho_min = $line_info[2];
        $rho_max = $line_info[3];
    } elsif ($line_info[0] eq "set") {
        $view = $line;
        $view_input = 1;
    } 
}

if ($first > $last || $last == 0) {
    printf STDERR "Problem with the first or last point check it!\n";
    printf OUTFILE "Problem with the first or last point check it!\n";
    exit;
}
if ($step == 0) { 
    printf OUTFILE "No step defined, default step = 1 chosen\n";
    $step = 1;
}
if ($name_input == 0) {
    printf STDERR "No input name specified, please modify\n";
    printf OUTFILE "No input name specified, please modify\n";
    exit;
}
if ($rep_input == 0) {
    printf OUTFILE "No representation specified, CPK representation will be used\n";
    $rep = "CPK";
}
if (($do_elf == 0) && ($do_nci == 0)) {
    printf OUTFILE "No ELF or NCI analysis asked, exit!\n";
    printf STDERR "No ELF or NCI analysis asked, exit!\n";
    exit;
} elsif (($do_elf == 1) && ($do_nci == 0)) {
    printf OUTFILE "ELF analysis only will be done\n";
} elsif (($do_elf == 0) && ($do_nci == 1)) {
    printf OUTFILE "NCI analysis only will be done\n";
} elsif (($do_elf == 1) && ($do_nci == 1)) {
    printf OUTFILE "ELF and NCI analysis will be done.\n";
}
if ($view_input == 0) {
    printf OUTFILE "No Viewpoint specified, exit!\n";
    printf STDERR "No Viewpoint specified, exit!\n";
    exit;
}



################################
#### Write surface vmd file ####
################################


printf OUTFILE "Start writing vmd surface file\n";


# Set settings
printf OUTPUT "#!/usr/local/bin/vmd\n";
printf OUTPUT "# VMD script written by save_state \$Revision: 1.41\n";
printf OUTPUT "# VMD version: 1.8.6\n";
printf OUTPUT "#variable\n";
printf OUTPUT "for {set i $first} {\$i<=$last} {incr i $step} {\n";
#printf OUTPUT " if {\$j<10} {\n";
#printf OUTPUT "  set i 00\$j}\n";
#printf OUTPUT " if {\$j>=10} {\n";
#printf OUTPUT "  set i 0\$j}\n";
#printf OUTPUT " if {\$j>=100} {\n";
#printf OUTPUT "  set i \$j}\n";
printf OUTPUT " #vmd\n";
printf OUTPUT "  set viewplist {}\n";
printf OUTPUT "# Display settings\n";
printf OUTPUT "  display projection   Orthographic\n" ;
printf OUTPUT "  display nearclip set 0.000000\n" ;
printf OUTPUT "  color scale method BGR\n" ;
printf OUTPUT "  set colorcmds {\n" ;
printf OUTPUT "  {color Element {C} black}\n" ;
printf OUTPUT "  {color Element {Cl} green}\n" ;
printf OUTPUT "  {color Element {F} yellow2}\n" ;
printf OUTPUT "  {color Element {O} red}\n" ;
printf OUTPUT "  {color Element {N} blue}\n" ;
printf OUTPUT "  {color Element {H} white}\n";
printf OUTPUT "  }\n" ;
printf OUTPUT "  foreach colcmd \$colorcmds {\n" ;
printf OUTPUT "  set val [catch {eval \$colcmd}]\n" ;
printf OUTPUT "  }\n" ;
printf OUTPUT "   color Display {Background} white\n" ;
printf OUTPUT " \n";


#print STDERR "$type\n";

# Load inputs for ELF/NCI
printf OUTPUT " # load new molecule elf and/or nci\n";

if (($do_elf == 1) && ($do_nci == 0)){
#	printf OUTPUT "  mol new $name\$\{i\}_esyn.cube         type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
	printf OUTPUT "  mol addfile $name\$\{i\}_elf.cube      type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
} 
elsif (($do_elf == 0) && ($do_nci == 1)){
	printf OUTPUT "  mol addfile $name\$\{i\}-dens.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
	printf OUTPUT "  mol addfile $name\$\{i\}-grad.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
}  
elsif (($do_elf == 1) && ($do_nci == 1)){
#        printf OUTPUT "  mol new $name\$\{i\}_esyn.cube         type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
        printf OUTPUT "  mol addfile $name\$\{i\}_elf.cube      type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
        printf OUTPUT "  mol addfile $name\$\{i\}-dens.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
        printf OUTPUT "  mol addfile $name\$\{i\}-grad.cube     type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
} 
else {
	print STDERR " No ELF or NCI analysis requested, please revise, exit!\n";
	exit;
}


#Set representation
printf OUTPUT " #representation of the atoms\n";
printf OUTPUT "  mol delrep 0 top\n";
if ($rep eq "CPK") {
	printf OUTPUT "  mol representation CPK 0.500000 0.100000 118.000000 131.000000\n";
}
elsif ($rep eq "line") {
	printf OUTPUT "  mol representation Line\n";
}
elsif ($rep eq "licorice") {
	printf OUTPUT "  mol representation Licorice 0.100000 70.000000 70.000000\n";
}
else {
	print STDERR " Revise Representation selection\n";
	print STDERR " Possibilities are CPK, line, licorice\n";
	exit;
}

printf OUTPUT "  mol color Element\n";
printf OUTPUT "  mol selection {all}\n";
printf OUTPUT "  mol material Opaque\n";
printf OUTPUT "  mol addrep top\n";


#Set ELF and NCI isosurfaces

if ($do_elf == 1) {
	if ($elf_cutoff == 0) {
                $elf_cutoff = 0.8;
                printf OUTFILE "No elf isovalue specified, 0.8 isovalue will be used\n";
        }
	printf OUTPUT " # add representation of the isosurface elf\n";
	printf OUTPUT "  mol representation Isosurface $elf_cutoff 0.000000 0.000000 0.000000 1 1\n";
	printf OUTPUT "  mol color ColorID 6\n";
	printf OUTPUT "  mol selection {all}\n"; 
	printf OUTPUT "  mol material Opaque\n"; 
	printf OUTPUT "  mol addrep top\n"; 
	printf OUTPUT "  mol selupdate 1 top 0\n"; 
	printf OUTPUT "  mol colupdate 1 top 0\n"; 
	printf OUTPUT "  mol scaleminmax top 1 0.000000 0.000000\n"; 
	printf OUTPUT "  mol smoothrep top 1 0\n"; 
	printf OUTPUT "  mol drawframes top 1 {now}\n"; 
	printf OUTPUT "  axes location off\n"; 
        printf OUTPUT "  display depthcue  off\n";
}


if ($do_nci == 1) {
	printf OUTPUT " # add representation of the isosurface nci\n";
        if ($rdg_cutoff == 0) {
                printf OUTFILE "Attention, no NCI isosurface cutoff defined, value chosen = 0.2\n";
		$rdg_cutoff = 0.2;
	}
	if (($rho_min == 0) && ($rho_max == 0)) {
		printf OUTFILE "Attention, no NCI color scale define, value chosen = -5.0; 5.0\n";
                $rho_min = -5;
                $rho_max = 5;
	}
	if ($rho_min > $rho_max) {
		printf OUTFILE "Problem in the definition of NCI color scale, exit!\n";
		exit;
	}
	if ($do_elf == 0) {
                printf OUTPUT "  mol representation Isosurface $rdg_cutoff 1.000000 0.000000 0.000000 1 1\n";
	        printf OUTPUT "  mol color Volume 0\n";
	        printf OUTPUT "  mol selection {all}\n"; 
	        printf OUTPUT "  mol material Opaque\n"; 
	        printf OUTPUT "  mol addrep top\n"; 
	        printf OUTPUT "  mol selupdate 1 top 0\n"; 
	        printf OUTPUT "  mol colupdate 1 top 0\n"; 
		printf OUTPUT "  mol scaleminmax top 1 $rho_min $rho_max\n";
	        printf OUTPUT "  mol smoothrep top 1 0\n"; 
	        printf OUTPUT "  mol drawframes top 1 {now}\n"; 
        }
        else {
                printf OUTPUT "  mol representation Isosurface $rdg_cutoff 2.000000 0.000000 0.000000 1 1\n";
                printf OUTPUT "  mol color Volume 1\n";
		printf OUTPUT "  mol selection {all}\n"; 
		printf OUTPUT "  mol material Opaque\n"; 
		printf OUTPUT "  mol addrep top\n"; 
		printf OUTPUT "  mol selupdate 2 top 0\n"; 
		printf OUTPUT "  mol colupdate 2 top 0\n"; 
	        printf OUTPUT "  mol scaleminmax top 2 $rho_min $rho_max\n";
		printf OUTPUT "  mol smoothrep top 2 0\n"; 
		printf OUTPUT "  mol drawframes top 2 {now}\n"; 
        }
	printf OUTPUT "  axes location off\n"; 
        printf OUTPUT "  display depthcue  off\n";
}


# Choose viewpoint
printf OUTPUT " #choose viewpoints\n";

printf OUTPUT "  $view\n";
printf OUTPUT "  lappend viewplist [molinfo top]\n";
printf OUTPUT "  set topmol [molinfo top]\n";

printf OUTPUT "    # done with molecule 0\n";
printf OUTPUT "   foreach v \$viewplist {\n";
printf OUTPUT "     molinfo \$v set {center_matrix rotate_matrix scale_matrix global_matrix} \$viewpoints(\$v)\n";
printf OUTPUT "     }\n";
printf OUTPUT "   unset viewplist\n";


# Output image
printf OUTPUT " #output image to a file\n";
printf OUTPUT "  render snapshot $name\$\{i\}_elf_nci.tga\n";
printf OUTPUT "  mol delete all\n";
printf OUTPUT "}\n";

close OUTPUT;

#####################################################################
# Write attractor VMD file
#####################################################################


printf OUTFILE "Start writing vmd attractor file\n";


# Set settings
printf OUTPUT2 "#!/usr/local/bin/vmd\n";
printf OUTPUT2 "# VMD script written by save_state \$Revision: 1.41\n";
printf OUTPUT2 "# VMD version: 1.8.6\n";
printf OUTPUT2 "#variable\n";
printf OUTPUT2 "for {set j $first} {\$j<=$last} {incr j $step} {\n";
printf OUTPUT2 " if {\$j<10} {\n";
printf OUTPUT2 "  set i 00\$j}\n";
printf OUTPUT2 " if {\$j>=10} {\n";
printf OUTPUT2 "  set i 0\$j}\n";
printf OUTPUT2 " if {\$j>=100} {\n";
printf OUTPUT2 "  set i \$j}\n";
printf OUTPUT2 " #vmd\n";
printf OUTPUT2 "  set viewplist {}\n";
printf OUTPUT2 "# Display settings\n";
printf OUTPUT2 "  display projection   Orthographic\n" ;
printf OUTPUT2 "  display nearclip set 0.000000\n" ;
printf OUTPUT2 "  color scale method BGR\n" ;
printf OUTPUT2 "  set colorcmds {\n" ;
printf OUTPUT2 "  {color Element {C} black}\n" ;
printf OUTPUT2 "  {color Element {Cl} green}\n" ;
printf OUTPUT2 "  {color Element {F} yellow2}\n" ;
printf OUTPUT2 "  {color Element {O} red}\n" ;
printf OUTPUT2 "  {color Element {N} blue}\n" ;
printf OUTPUT2 "  {color Element {H} white}\n";
printf OUTPUT2 "  }\n" ;
printf OUTPUT2 "  foreach colcmd \$colorcmds {\n" ;
printf OUTPUT2 "  set val [catch {eval \$colcmd}]\n" ;
printf OUTPUT2 "  }\n" ;
printf OUTPUT2 "   color Display {Background} white\n" ;
printf OUTPUT2 " axes location off\n";
printf OUTPUT2 " \n";

# Load NCI Critical Point of Interaction
printf OUTPUT2 " # load new molecule\n";
printf OUTPUT2 " mol new $name\$\{i\}.wfn_topo.xyz         type xyz first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";

#Set representation
printf OUTPUT2 " #representation of the atoms\n";
printf OUTPUT2 "  mol delrep 0 top\n";
if ($rep eq "CPK") {
	printf OUTPUT2 "  mol representation CPK 0.500000 0.100000 118.000000 131.000000\n";
}
elsif ($rep eq "line") {
	printf OUTPUT2 "  mol representation Line\n";
}
elsif ($rep eq "licorice") {
	printf OUTPUT2 " mol representation Licorice 0.100000 70.000000 70.000000\n";
}
else {
	print STDERR " Revise Representation selection\n";
	print STDERR " Possibilities are CPK, line, licorice\n";
	exit;
}

printf OUTPUT2 "  mol color Element\n";
printf OUTPUT2 "  mol selection {all}\n";
printf OUTPUT2 "  mol material Opaque\n";
printf OUTPUT2 "  mol addrep top\n";
printf OUTPUT2 " # add representation of the interaction points\n"; 
printf OUTPUT2 "  mol representation CPK 0.200000 0.000000 118.000000 0.000000\n"; 
printf OUTPUT2 "  mol color Name\n"; 
printf OUTPUT2 "  mol selection {name X}\n"; 
printf OUTPUT2 "  mol material Opaque\n"; 
printf OUTPUT2 "  mol addrep top\n"; 

printf OUTPUT2 " # choose viewpoints\n";
printf OUTPUT2 " $view\n";
printf OUTPUT2 "  lappend viewplist [molinfo top]\n";
printf OUTPUT2 "  set topmol [molinfo top]\n";
printf OUTPUT2 " # done with molecule 0\n";

printf OUTPUT2 " # load new molecule\n";
printf OUTPUT2 "  mol new $name\$\{i\}_ebas.xyz             type xyz first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all\n";
printf OUTPUT2 " # add representation of the attractors\n"; 
printf OUTPUT2 " mol delrep 0 top\n"; 
printf OUTPUT2 " mol representation CPK 0.200000 0.000000 118.000000 0.000000\n"; 
printf OUTPUT2 " mol color Element\n"; 
printf OUTPUT2 " mol selection {name xx}\n"; 
printf OUTPUT2 " mol material Opaque\n"; 
printf OUTPUT2 " mol addrep top\n"; 

printf OUTPUT2 " # choose viewpoints\n";
printf OUTPUT2 " $view\n";
printf OUTPUT2 "  lappend viewplist [molinfo top]\n";
printf OUTPUT2 "  set topmol [molinfo top]\n";
printf OUTPUT2 " # done with molecule 1\n";

printf OUTPUT2 " foreach v \$viewplist {\n"; 
printf OUTPUT2 "    molinfo \$v set {center_matrix rotate_matrix scale_matrix global_matrix} \$viewpoints(\$v)\n"; 
printf OUTPUT2 "    }\n"; 
printf OUTPUT2 " unset viewplist\n"; 
printf OUTPUT2 " #output image to a file\n"; 
printf OUTPUT2 " render snapshot $name\$\{i\}_topo.tga\n"; 
printf OUTPUT2 " mol delete all\n"; 
printf OUTPUT2 "}\n"; 

close OUTPUT2;




printf OUTFILE "vmd files created, exiting\n";

close OUTFILE;

